@@ -401,6 +401,11 @@ void GetFuncInfo (const char* Name, unsigned short* Use, unsigned short* Chg)
             */
             if ((D->Flags & FD_VARIADIC) != 0) {
                 *Use = REG_Y;
+            } else if (D->Flags & FD_CALL_WRAPPER) {
+                /* Wrappers may go to any functions, so mark them as using all
+                ** registers.
+                */
+                *Use = REG_EAXY;
             } else if (D->ParamCount > 0 &&
                        (AutoCDecl ?
                         IsQualFastcall (E->Type) :
