@@ -20,7 +20,7 @@ int main(void) {
     n = (p == &func);
     n = (p == func);
 
-    n = func - p; /* invalid C */
+    n = p - func; /* invalid C */
 
     return 0;
 }
